## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
from CvPythonExtensions import *
import CvUtil
import ScreenInput
import CvScreenEnums
import string
import CvScreensInterface

#Change CvScreenEnums to include CvPromotionScreen (26)
#We will need to have code that shows the gold cost for the total promotion changes.

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

class CvPromotionsScreen:
	"Promotions Screen"

	def __init__(self):
		self.SCREEN_NAME = "PromotionsScreen"
		self.CANCEL_NAME = "PromotionsCancel"
		self.EXIT_NAME = "PromotionsExit"
		self.TITLE_NAME = "PromotionsTitleHeader"
		self.BUTTON_NAME = "PromotionsScreenButton"
		self.TEXT_NAME = "PromotionsScreenText"
		self.AREA_NAME = "PromotionsScreenArea"
		self.HELP_AREA_NAME = "PromotionsScreenHelpArea"
		self.HELP_IMAGE_NAME = "PromotionsScreenPromotionClassImage"
		self.DEBUG_DROPDOWN_ID =  "PromotionsDropdownWidget"
		self.BACKGROUND_ID = "PromotionsBackground"
		self.HELP_HEADER_NAME = "PromotionsScreenHeaderName"

		self.HEADINGS_WIDTH = 199
		self.HEADINGS_TOP = 70
		self.HEADINGS_SPACING = 5
		self.HEADINGS_BOTTOM = 280
		self.HELP_TOP = 350
		self.HELP_BOTTOM = 610
		self.TEXT_MARGIN = 15
		self.BUTTON_SIZE = 24
		self.BIG_BUTTON_SIZE = 64
		self.BOTTOM_LINE_TOP = 630
		self.BOTTOM_LINE_WIDTH = 1014
		self.BOTTOM_LINE_HEIGHT = 60

		self.X_EXIT = 600
		self.Y_EXIT = 726

		self.X_CANCEL = 400
		self.Y_CANCEL = 726

		self.X_SCREEN = 800
		self.Y_SCREEN = 400
		self.W_SCREEN = 600
		self.H_SCREEN = 800
		self.Z_SCREEN = -6.1
		self.Y_TITLE = 8		
		self.Z_TEXT = self.Z_SCREEN - 0.2

		self.PromotionsScreenInputMap = {
			self.BUTTON_NAME		: self.PromotionsButton,
			self.TEXT_NAME			: self.PromotionsButton,
			self.EXIT_NAME			: self.SetPromotions,
			self.CANCEL_NAME		: self.Cancel,
			}

		self.iActivePlayer = -1
		
		self.m_pabOriginalPromotions = []
		self.m_pabDisplayPromotions = []
		self.m_pabCurrentPromotions = []
		self.m_paiFilledPromotionClassSlots = []
		
	def getScreen(self):
		return CyGInterfaceScreen(self.SCREEN_NAME, CvScreenEnums.PROMOTIONS_SCREEN)

	def setActivePlayer(self, iPlayer):

		self.iActivePlayer = iPlayer
		activePlayer = gc.getPlayer(iPlayer)

		self.m_pabCurrentPromotions = []
		self.m_pabDisplayPromotions = []
		self.m_pabOriginalPromotions = []
		self.m_paiFilledPromotionClassSlots = []
		for i in range (gc.getNumPromotionInfos()):
			self.m_pabCurrentPromotions.append(CyInterface().getHeadSelectedUnit().isHasPromotion(i));
			self.m_pabDisplayPromotions.append(CyInterface().getHeadSelectedUnit().isHasPromotion(i));
			self.m_pabOriginalPromotions.append(CyInterface().getHeadSelectedUnit().isHasPromotion(i));
		#for i in range (gc.getNumPromotionClassInfos()):
		#	self.m_paiFilledPromotionClassSlots.append(CyInterface().getHeadSelectUnit().getFilledPromotionClassSlots(i));
			

	def interfaceScreen (self):

		screen = self.getScreen()
		if screen.isActive():
			return
		screen.setRenderInterfaceOnly(True);
		screen.showScreen( PopupStates.POPUPSTATE_IMMEDIATE, False)
	
		# Set the background and exit button, and show the screen
		screen.setDimensions(screen.centerX(0), screen.centerY(0), self.W_SCREEN, self.H_SCREEN)
		screen.addDDSGFC(self.BACKGROUND_ID, ArtFileMgr.getInterfaceArtInfo("MAINMENU_SLIDESHOW_LOAD").getPath(), 0, 0, self.W_SCREEN, self.H_SCREEN, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.addPanel( "TechTopPanel", u"", u"", True, False, 0, 0, self.W_SCREEN, 55, PanelStyles.PANEL_STYLE_TOPBAR )
		screen.addPanel( "TechBottomPanel", u"", u"", True, False, 0, 713, self.W_SCREEN, 55, PanelStyles.PANEL_STYLE_BOTTOMBAR )
		screen.showWindowBackground(False)
		screen.setText(self.CANCEL_NAME, "Background", u"<font=4>" + localText.getText("TXT_KEY_SCREEN_CANCEL", ()).upper() + u"</font>", CvUtil.FONT_CENTER_JUSTIFY, self.X_CANCEL, self.Y_CANCEL, self.Z_TEXT, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, 1, 0)

		# Header...
		screen.setText(self.TITLE_NAME, "Background", u"<font=4b>" + localText.getText("TXT_KEY_PROMOTIONS_SCREEN_TITLE", ()).upper() + u"</font>", CvUtil.FONT_CENTER_JUSTIFY, self.X_SCREEN, self.Y_TITLE, self.Z_TEXT, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
		
		self.setActivePlayer(gc.getGame().getActivePlayer())						

		screen.addPanel("PromotionsBottomLine", "", "", True, True, self.HEADINGS_SPACING, self.BOTTOM_LINE_TOP, self.BOTTOM_LINE_WIDTH, self.BOTTOM_LINE_HEIGHT, PanelStyles.PANEL_STYLE_MAIN)

		# Draw Contents
		self.drawContents()

		return 0
		
	# Draw the contents...
	def drawContents(self):
	
		# Draw the check box buttons
		self.drawAllButtons()
				
		# Draw Help Text
		self.drawAllHelpText()
		
		# Update Maintenance/anarchy/etc.
		self.updateCost()
		
	def drawPromotionClassButtons(self):

		activePlayer = gc.getPlayer(self.iActivePlayer)
		screen = self.getScreen()
		
		for j in range(gc.getNumPromotionInfos()):
			screen.setState(self.getPromotionsButtonName(j), self.m_paeCurrentPromotions[iPromotion])
							
			if (self.m_pabDisplayPromotions[j]):
				screen.show(self.getPromotionsButtonName(j))
			#elif (activePlayer.canDoCivics(j)):
			#	screen.show(self.getPromotionsButtonName(j))
			else:
				screen.hide(self.getPromotionsButtonName(j))
								
	# Will draw the radio buttons (and revolution)
	def drawAllButtons(self):				

		for i in range(gc.getNumPromotionClassInfos()):
		
			fX = self.HEADINGS_SPACING  + (self.HEADINGS_WIDTH + self.HEADINGS_SPACING) * i
			fY = self.HEADINGS_TOP
			szAreaID = self.AREA_NAME + str(i)
			screen = self.getScreen()
			screen.addPanel(szAreaID, "", "", True, True, fX, fY, self.HEADINGS_WIDTH, self.HEADINGS_BOTTOM - self.HEADINGS_TOP, PanelStyles.PANEL_STYLE_MAIN)
			screen.setLabel("", "Background",  u"<font=3>" + gc.getPromotionClassInfo(i).getDescription().upper() + u"</font>", CvUtil.FONT_CENTER_JUSTIFY, fX + self.HEADINGS_WIDTH/2, self.HEADINGS_TOP + self.TEXT_MARGIN, 0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )

			fY += self.TEXT_MARGIN
			
			for j in range(gc.getNumPromotionInfos()):
				if (gc.getPromotionInfo(j).getType() == i):										
					fY += 2 * self.TEXT_MARGIN
					screen.addCheckBoxGFC(self.getPromotionsButtonName(j), gc.getPromotionInfo(j).getButton(), ArtFileMgr.getInterfaceArtInfo("BUTTON_HILITE_SQUARE").getPath(), fX + self.BUTTON_SIZE/2, fY, self.BUTTON_SIZE, self.BUTTON_SIZE, WidgetTypes.WIDGET_HELP_PROMOTION, -1, -1, ButtonStyles.BUTTON_STYLE_LABEL)
					screen.setText(self.getPromotionTextName(j), "", gc.getPromotionInfo(j).getDescription(), CvUtil.FONT_LEFT_JUSTIFY, fX + self.BUTTON_SIZE + self.TEXT_MARGIN, fY, 0, FontTypes.SMALL_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)

			self.drawPromotionClassButtons(i)

	def highlight(self, iPromotion):
		if ( not self.m_pabDisplayPromotions[iPromotion]):
			self.m_pabDisplayPromotions[iPromotion] = True
			self.drawPromotionClassButtons(iPromotion)
			return True
		return False
		
	def unHighlight(self, iPromotion):		
		if self.m_paeDisplayPromotion[iPromotion] != self.m_pabCurrentPromotions[iPromotion]:
			self.m_pabDisplayPromotions[iPromotion] = self.m_pabCurrentPromotions[iPromotion]
			self.drawPromotionClassButtons(iPromotion)
			return True
		return False
		
	def select(self, iPromotion):
		activePlayer = gc.getPlayer(self.iActivePlayer)
		if (not activePlayer.canDoCivics(iCivic)):
			# If you can't even do this, get out....
			return 0
			
		iPromotionClass = gc.getPromotionInfo(iPromotion).getPromotionClassType()
		
		
		if (CyInterface().getHeadSelectedUnit().getMaxPromotionSlots() == 1):
			for i in range (gc.getNumPromotionInfos()):
				if (self.m_pabCurrentPromotions[i] and (gc.getPromotionInfo(i).getPromotionClassType() == iPromotionClass)):
					
			#Returns a zero if you
					return 0
		# Set the previous widget
		iCivicPrev = self.m_pabCurrentPromotions[iCivicOption]
		
		# Switch the widgets
		self.m_pabCurrentPromotion[iPromotion] = True
		
		# Unighlight the previous widget
		self.unHighlight(iPromotionPrev)
		self.getScreen().setState(self.getPromotionsButtonName(iPromotionPrev), False)

		# highlight the new widget
		self.highlight(iPromotion)		
		self.getScreen().setState(self.getPromotionsButtonName(iPromotion), True)
		
		return 0

	def PromotionsButton(self, inputClass):
	
		if (inputClass.getNotifyCode() == NotifyCode.NOTIFY_CLICKED) :
			if (inputClass.getFlags() & MouseFlags.MOUSE_RBUTTONUP):
				CvScreensInterface.pediaJumpToPromotion((inputClass.getID(), ))
			else:
				# Select button
				self.select(inputClass.getID())
				self.drawHelpText(gc.getPromotionInfo(inputClass.getID()).getPromotionClassType())
				self.updateCost()
		elif (inputClass.getNotifyCode() == NotifyCode.NOTIFY_CURSOR_MOVE_ON) :
			# Highlight this button
			if self.highlight(inputClass.getID()):
				self.drawHelpText(gc.getPromotionInfo(inputClass.getID()).getPromotionClassType())
		elif (inputClass.getNotifyCode() == NotifyCode.NOTIFY_CURSOR_MOVE_OFF) :
			if self.unHighlight(inputClass.getID()):
				self.drawHelpText(gc.getPromotionInfo(inputClass.getID()).getPromotionClassType())
				self.updateCost()

		return 0

		
	def drawHelpText(self, iPromotion):
		
		activePlayer = gc.getPlayer(self.iActivePlayer)
		iCivic = self.m_pabDisplayPromotions[iPromotion]

		szPaneID = "PromotionsHelpTextBackground" + str(iPromotion)
		screen = self.getScreen()

		szHelpText = u""

		# Upkeep string
		#if ((gc.getPromotionInfo(iPromotion).getUpkeep() != -1) and not activePlayer.isNoCivicUpkeep(iCivicOption)):
		#	szHelpText = gc.getUpkeepInfo(gc.getCivicInfo(iCivic).getUpkeep()).getDescription()
		#else:
		#	szHelpText = localText.getText("TXT_KEY_CIVICS_SCREEN_NO_UPKEEP", ())

		szHelpText += CyGameTextMgr().parseCivicInfo(iCivic, False, True, True)

		fX = self.HEADINGS_SPACING  + (self.HEADINGS_WIDTH + self.HEADINGS_SPACING) * iPromotion

		screen.setLabel(self.HELP_HEADER_NAME + str(iPromotion), "Background",  u"<font=3>" + gc.getPromotionInfo(iPromotion).getDescription().upper() + u"</font>", CvUtil.FONT_CENTER_JUSTIFY, fX + self.HEADINGS_WIDTH/2, self.HELP_TOP + self.TEXT_MARGIN, 0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )

		fY = self.HELP_TOP - self.BIG_BUTTON_SIZE
		szHelpImageID = self.HELP_IMAGE_NAME + str(iPromotion)		
		screen.setImageButton(szHelpImageID, gc.getCivicInfo(iPromotion).getButton(), fX + self.HEADINGS_WIDTH/2 - self.BIG_BUTTON_SIZE/2, fY, self.BIG_BUTTON_SIZE, self.BIG_BUTTON_SIZE, WidgetTypes.WIDGET_PEDIA_JUMP_TO_PROMOTION, iPromotion, 1)

		fY = self.HELP_TOP + 3 * self.TEXT_MARGIN
		szHelpAreaID = self.HELP_AREA_NAME + str(iPromotion)		
		screen.addMultilineText(szHelpAreaID, szHelpText, fX+5, fY, self.HEADINGS_WIDTH-7, self.HELP_BOTTOM - fY-2, WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)				
		
		
	# Will draw the help text
	def drawAllHelpText(self):
		for i in range (gc.getNumCivicOptionInfos()):		

			fX = self.HEADINGS_SPACING  + (self.HEADINGS_WIDTH + self.HEADINGS_SPACING) * i

			szPaneID = "CivicsHelpTextBackground" + str(i)
			screen = self.getScreen()
			screen.addPanel(szPaneID, "", "", True, True, fX, self.HELP_TOP, self.HEADINGS_WIDTH, self.HELP_BOTTOM - self.HELP_TOP, PanelStyles.PANEL_STYLE_MAIN)

			self.drawHelpText(i)

	
	def updateCost(self):
	
		for i in range (gc.getNumPromotionInfos()):
			if (self.m_paeCurrentPromotions[i] != self.m_paeOriginalPromotions[i]):
				iCost += gc.getPromotionInf(i).getGoldCost()
				
		szText = localText.getText("TXT_KEY_ANARCHY_TURNS", (iTurns, ))

		screen.setLabel("PromotionCostText", "Background", u"<font=3>" + szText + u"</font>", CvUtil.FONT_CENTER_JUSTIFY, self.X_SCREEN, self.BOTTOM_LINE_TOP + self.TEXT_MARGIN//2, 0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
	
	
	def SetPromotions(self, inputClass):
	
		pHeadSelectedUnit = CyInterface().getHeadSelectedUnit()
		
		for i in range (gc.getNumPromotionInfos()):
			pHeadSelectedUnit.setHasPromotion(i, m_pabCurrentPromotions[i])
		
	def Cancel(self, inputClass):
		screen = self.getScreen()
		if (inputClass.getNotifyCode() == NotifyCode.NOTIFY_CLICKED) :
			for i in range (gc.getNumPromotionInfos()):
				self.m_pabCurrentPromotions[i] = self.m_pabOriginalPromotions[i]
				self.m_pabDisplayPromotions[i] = self.m_pabOriginalPromotions[i]
			
			self.drawContents()
			
	def getPromotionsButtonName(self, iPromotion):
		szName = self.BUTTON_NAME + str(iPromotion)
		return szName

	def getPromotionsTextName(self, iPromotion):
		szName = self.TEXT_NAME + str(iPromotion)
		return szName

	# Will handle the input for this screen...
	def handleInput(self, inputClass):
		if (inputClass.getNotifyCode() == NotifyCode.NOTIFY_LISTBOX_ITEM_SELECTED):
			screen = self.getScreen()
			iIndex = screen.getSelectedPullDownID(self.DEBUG_DROPDOWN_ID)
			self.setActivePlayer(screen.getPullDownData(self.DEBUG_DROPDOWN_ID, iIndex))
			self.drawContents()
			return 1
		elif (self.PromotionsScreenInputMap.has_key(inputClass.getFunctionName())):	
			'Calls function mapped in CvCivicsScreen'
			# only get from the map if it has the key		

			# get bound function from map and call it
			self.PromotionsScreenInputMap.get(inputClass.getFunctionName())(inputClass)
			return 1
		return 0
		
	def update(self, fDelta):
		return