## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
## 
## This class is passed an argsList from CvAppInterface.onEvent
## The argsList can contain anything from mouse location to key info
## The EVENTLIST that are being notified can be found 

from CvPythonExtensions import *
import CvUtil
import CvScreensInterface
import CvDebugTools
import CvWBPopups
import PyHelpers
import Popup as PyPopup
import CvCameraControls
import CvTopCivs
import CvEventManager
import sys
import SpaceMod

gc = CyGlobalContext()
localText = CyTranslator()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo


# globals
###################################################
class CvCustomEventManager(CvEventManager.CvEventManager):
	def __init__(self):
		# initialize base class
		self.parent = CvEventManager.CvEventManager
		self.parent.__init__(self)
		self.spaceMod = SpaceMod.SpaceMod()
		self.CustomEvents={
            5001 : ('UnitNameBox', self.__eventShinPopup2Apply, self.__eventShinPopup2Begin)
        }

	def onCityBuilt(self, argsList):
		self.parent.onCityBuilt(self, argsList)
		
	def onKbdEvent(self, argsList):
		'keypress handler - return 1 if the event was consumed'

		eventType,key,mx,my,px,py = argsList
		game = CyGame()
		
		if (self.bAllowCheats):
			# notify debug tools of input to allow it to override the control
			argsList = (eventType,key,self.bCtrl,self.bShift,self.bAlt,mx,my,px,py,CyGame().isNetworkMultiPlayer())
			if ( CvDebugTools.g_CvDebugTools.notifyInput(argsList) ):
				return 0
		
		if ( eventType == self.EventKeyDown ):
			theKey=int(key)
			
			CvCameraControls.g_CameraControls.handleInput( theKey )
						
			# Shift - T (Debug - No MP)
			if (self.bAllowCheats and theKey == int(InputTypes.KB_T)):
				if ( self.bShift ):
					self.beginEvent(CvUtil.EventAwardTechsAndGold)
					#self.beginEvent(CvUtil.EventCameraControlPopup)
					return 1
						
			# Shift - ] (Debug - currently mouse-overd unit, health += 10
			elif (self.bAllowCheats and theKey == int(InputTypes.KB_LBRACKET) and self.bShift ):
				unit = CyMap().plot(px, py).getUnit(0)
				if ( not unit.isNone() ):
					d = min( unit.maxHitPoints()-1, unit.getDamage() + 10 )
					unit.setDamage( d, PlayerTypes.NO_PLAYER )
				
			# Shift - [ (Debug - currently mouse-overd unit, health -= 10
			elif (self.bAllowCheats and theKey == int(InputTypes.KB_RBRACKET) and self.bShift ):
				unit = CyMap().plot(px, py).getUnit(0)
				if ( not unit.isNone() ):
					d = max( 0, unit.getDamage() - 10 )
					unit.setDamage( d, PlayerTypes.NO_PLAYER )
				
			elif (self.bAllowCheats and theKey == int(InputTypes.KB_F1)):
				if ( self.bShift ):
					CvScreensInterface.replayScreen.showScreen(False)
					return 1
				# don't return 1 unless you want the input consumed
			
			elif (self.bAllowCheats and theKey == int(InputTypes.KB_F2)):
				if ( self.bShift ):
					import CvDebugInfoScreen
 					CvScreensInterface.showDebugInfoScreen()
					return 1
			
			elif (self.bAllowCheats and theKey == int(InputTypes.KB_F3)):
				if ( self.bShift ):
					CvScreensInterface.showDanQuayleScreen(())
					return 1
					
			elif (self.bAllowCheats and theKey == int(InputTypes.KB_F4)):
				if ( self.bShift ):
					CvScreensInterface.showUnVictoryScreen(())
					return 1
					
			## vadus - test screen - begin
			elif (theKey == int(InputTypes.KB_U)) and (self.bShift):
				self.spaceMod.showUnitBoxScreen()
				return 1
			## vadus - test screen - end
	
	def onUnitCreated(self, argsList):
		'Unit Completed'
		self.parent.onUnitCreated(self, argsList)
		pUnit = argsList[0]
		#self.spaceMod.promoteUnit(pUnit)
		
	def onUnitBuilt(self, argsList):
		'Unit Completed'
		self.parent.onUnitBuilt(self, argsList)
		self.spaceMod.onUnitBuilt(argsList)
		#this function has to be used later
		
	def onCityBuildingUnit(self, argsList):
		'City begins building a unit'
		self.parent.onCityBuildingUnit(self, argsList)
		
			
	def beginEvent( self, context, argsList=-1 ):
		"Begin Event"
		#entry = self.Events[context]
		#return entry[2]( argsList )
		if(self.CustomEvents.has_key(context)):
			return self.CustomEvents[context][2](argsList)
		else:
			self.parent.beginEvent(self, context, argsList)
            
	def applyEvent( self, argsList ):
		'Apply the effects of an event '
		context, playerID, netUserData, popupReturn = argsList
		
		if(self.CustomEvents.has_key(context)):
			entry = self.CustomEvents[context]
			return entry[1]( playerID, netUserData, popupReturn )   # the apply function
		else:
			return self.parent.applyEvent(self, argsList)
		
	def __eventShinPopup2Begin(self, iUnitName):
		pass
           
	def __eventShinPopup2Apply(self, playerID, netUserData, popupReturn):
		pPlayer = gc.getPlayer(0)
		iX = pPlayer.getStartingPlot().getX()
		iY = pPlayer.getStartingPlot().getY()
		message = "In __eventShinPopup2Apply(%s)" % (popupReturn.getEditBoxString(0))
		CyInterface().addImmediateMessage(message, "")
		self.spaceMod.setCurrentUnitName(popupReturn.getEditBoxString(0))
		#CyInterface().addMessage(0,True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iX,iY,True,True)
        
				