# UnitBoxScreen.py
# Author : Vadus
# This is the Class for the UnitSandbox Screen . For creating this, a sample form the 
# RandomEventsMod was taken

from CvPythonExtensions import *
import CvScreenEnums
import PyHelpers
import CvUtil
import ScreenInput
import Popup as PyPopup
import SpaceMod
import VadusUnit

gc = CyGlobalContext()

class UnitBoxScreen:
    
    def __init__(self, screenId):
        self.screenQueue = []   #queue for showing screens
        self.screenId = screenId
        self.screen = None
        self.WIDGET_ID = "UnitBoxScreenWidget"   #used to set widget ID strings so that widgets can be easily created and deleted
        self.SCREEN_NAME = "UnitBoxScreen"
        self.nWidgetCount = 0    #used to keep a count of all widgets created, so they can be deleted again after

        #formatting constants
        self.xcentre = 0
        self.ycentre = 0
        self.WIDTH = 800
        self.HEIGHT = 600
        
        
        #additional attributes
        self.spaceMod = None
        self.screenValues = {"screenOpen" : False,
                             "weaponChanged" : False, 
                             "selectedClass" : None,
                             "selectedUnit" : None,
                             "selectedSpecials" : None,
                             "currentUnitName" : None,
                             "promotionSelected" : -1
                              }
        #Promotions attributes
        self.unitPromotionList = []
        # maxPromotions should be not more than 6 !
        self.maxPromotions = 4                      
        self.unitPromotions = []
        for i in range(self.maxPromotions):
            self.unitPromotions.append(-1)
        
        #this directory is important for the appliedUnitButtons and unitPromotionButtons, because a widgedName seems to must not contain a number, otherwise GUI problems will occur ;-( ,  so each number becomes a char ;-)
        self.unitChars = {}
        for i in range(65, 123):
            self.unitChars[i] = chr(i)
        
        # list with items for the left upper drop down box
        self.unitClasses = []
        
        # list with items for the left middle drop down box
        self.weaponList = []
              
#        self.Events={ 5001 : ('EditUnitName', self.__eventEditUnitNameApply, self.__eventEditUnitNameBegin) }
        
    def showScreen(self, parentMod):
        self.spaceMod = parentMod
        self.fillUnitClasses()
        self.fillWeaponList()
        "shows the popup"
#        CyInterface().addImmediateMessage("in showScreen() of UnitBoxScreen","")
        #only show the popup if there are popups queued to be shown
        
        if (len(self.screenQueue) != 0):
        
                if ( CyGame().isPitbossHost() ):
                    return
        
                # Create screen
        
                self.screen = self.getScreen()
        
                #show the screen on the monitor
        
                self.screen.showScreen(PopupStates.POPUPSTATE_QUEUED, False)
        
                #make window invisible except for other drawn widgets

                self.screen.showWindowBackground( False )

                #set popup dimensions - make it cover the whole screen so user cant click elsewhere
                
                self.screen.setDimensions(0, 0, self.screen.getXResolution(), self.screen.getYResolution())

                #set the screen resolution values to get the centre of the screen to show widgets properly formatted

                self.xcentre = self.screen.getXResolution() / 2
                self.ycentre = self.screen.getYResolution() / 2
                
                # create the main panel. I believe PanelStyles.PANEL_STYLE_MAIN gives the dialog box the rounded blue background
        
                self.screen.addPanel( "UnitBoxScreenPanel", "", "", true, true, self.xcentre - (self.WIDTH/2) , self.ycentre - (self.HEIGHT/2), self.WIDTH , self.HEIGHT, PanelStyles.PANEL_STYLE_MAIN )
                
                #create Apply Button
                
                self.screen.setButtonGFC("Apply", "Apply", "", self.xcentre + self.WIDTH/10, self.ycentre + (self.HEIGHT/2) - 50, (self.WIDTH/20)*6, 40, WidgetTypes.WIDGET_GENERAL, -1, -1, ButtonStyles.BUTTON_STYLE_STANDARD )
                
                #create the Cancel button
                
                self.screen.setButtonGFC("Exit", "Leave", "", self.xcentre - (self.WIDTH/20)*8 , self.ycentre + (self.HEIGHT/2) - 50, (self.WIDTH/20)*6, 40, WidgetTypes.WIDGET_GENERAL, -1, -1, ButtonStyles.BUTTON_STYLE_STANDARD )

                #show the first screen in the queue            

                self.showNextScreen()
                
    def handleInput( self, inputClass ):
        "WHEN A SCREEN IS UP, ALL SCREEN EVENTS ARE SENT TO ITS HANDLE INPUT FUNCTION. It continues to send all events to the screen until the screen hides itself"
        #CyInterface().addImmediateMessage("any Button pressed","")
#        CvUtil.pyPrint("notify code is:  " + str(inputClass.getNotifyCode()))
#        CvUtil.pyPrint("uiflags are:  " + str(inputClass.getFlags()))
#        CvUtil.pyPrint("getData are:  " + str(inputClass.getData()))
#        CvUtil.pyPrint("iItemID are:  " + str(inputClass.getID()))
#        CvUtil.pyPrint("getPythonFile are:  " + str(inputClass.getPythonFile()))
#        CvUtil.pyPrint("getFunctionName are:  " + str(inputClass.getFunctionName()))
#        CvUtil.pyPrint("getData1 are:  " + str(inputClass.getData1()))
#        CvUtil.pyPrint("getData2 are:  " + str(inputClass.getData2()))
#        CvUtil.pyPrint("getOption are:  " + str(inputClass.getOption()))
#        CvUtil.pyPrint("Input Class: ")
#        CvUtil.pyPrint(str(inputClass))

        morescreens = false

        #big if condition means: the user can either click on the ok button or they can press return or numpad enter to close popup
        #note - itemID is 1 for keypress down and 0 for keypress up
        if ( (inputClass.getFunctionName() == "Apply" and inputClass.getNotifyCode() == NotifyCode.NOTIFY_CLICKED) or ((inputClass.getData() == int(InputTypes.KB_RETURN) or inputClass.getData() == int(InputTypes.KB_NUMPADENTER)) and inputClass.getID() == 1 ) ):
            #CyInterface().addImmediateMessage("ok Button pressed","")
            promos = []
            for promo in self.unitPromotions:
                if (promo != -1):
                    promos.append(promo)
            self.spaceMod.addUnit(self.screenValues["selectedClass"], self.screenValues["selectedUnit"], self.screenValues["currentUnitName"], *promos)
            self.updateScreen(self.screenId)
            
            morescreens = self.showNextScreen()

            return 1
        
        if ( (inputClass.getFunctionName() == "Exit" and inputClass.getNotifyCode() == NotifyCode.NOTIFY_CLICKED) or ((inputClass.getData() == int(InputTypes.KB_RETURN) or inputClass.getData() == int(InputTypes.KB_NUMPADENTER)) and inputClass.getID() == 1 ) ):

            #delete all widgets and show the next queued screen if there is one, otherwise hide the screen

            self.deleteAllWidgets()
            
            morescreens = self.showNextScreen()

            if (morescreens == false):
                self.deleteAllWidgets()
                self.screenValues["screenOpen"] = False 
                self.screenValues["weaponChanged"] = False   
                self.screen.hideScreen()

                "return 1 to consume the input - this tells the framework that this screen was able to handle the event"
                return 1
            else:
                return 1
        # if selection of UnitClassChoice was changed
        if inputClass.getFunctionName() == "UnitClassChoice":
            selectionIndex = self.screen.getSelectedPullDownID("UnitClassChoice")
#            CyInterface().addImmediateMessage("drop down changed to %d - %s" %(selectionIndex, self.weaponList[self.screen.getSelectedPullDownID("UnitClassChoice")]),"")
#            CyInterface().addImmediateMessage("dropDownEvent : selecting now %d , len(unitClasses):%d" %(selectionIndex, len(self.unitClasses) ), "")
            self.screenValues["selectedClass"] = self.unitClasses[selectionIndex]
            self.fillWeaponList()
            self.screenValues["selectedUnit"] = self.weaponList[len(self.weaponList) -1]
            self.screenValues["weaponChanged"] = False
            self.screenValues["currentUnitName"] = None
            self.resetPromotions()
            self.updateScreen(self.screenId)
            return 1
        #if selection of weapon was changed
        if inputClass.getFunctionName() == "UnitWeaponChoice":
            selectionIndex = self.screen.getSelectedPullDownID("UnitWeaponChoice")
#            CyInterface().addImmediateMessage("drop down changed to %d - %s" %(selectionIndex, self.weaponList[self.screen.getSelectedPullDownID("UnitClassChoice")]),"")
#            CyInterface().addImmediateMessage("dropDownEvent : selecting now %d , len(unitClasses):%d" %(selectionIndex, len(self.unitClasses) ), "")
            self.screenValues["selectedUnit"] = self.weaponList[selectionIndex]
            self.resetPromotions()
            self.updateScreen(self.screenId)
            return 1
        # if UnitNameText was clicked
        elif inputClass.getFunctionName() == "UnitNameBox":
#            CyInterface().addImmediateMessage("clicked on UnitNameBox", "")
            self.__eventEditUnitRename(inputClass.getData1())
            return 1
        
        # if an item of the promo list was pressed
        elif inputClass.getFunctionName() == "PromotionBoxList":
            #CyInterface().addImmediateMessage("clicked on %s"%(gc.getPromotionInfo(inputClass.getData1()).getDescription()), "")
            for i in range(self.maxPromotions):
                if (i == self.screenValues["promotionSelected"]):
                    self.unitPromotions[i] = inputClass.getData1()
            self.updateUnitPromotionList()
            self.updateScreen(self.screenId)
            return 1
        
        
        #unfortunately to add a number the functions string variable will make the buttons flickering on the GUI
        #like: if inputClass.getFunctionName() == "appliedUnitButton%d"%(inputClass.getData1()): bla
        #so that I have to use the alphabet... not optimal ;-(
        
        elif inputClass.getData1() > 64 and inputClass.getData1() < 123:
            
            #if an appliedUnitButton was pressed
            #CyInterface().addImmediateMessage("inputClass.getData1 = %d"%(inputClass.getData1()), "")
            if inputClass.getFunctionName() == "appliedUnitButton%s"%(self.unitChars[inputClass.getData1()]):
                #CyInterface().addImmediateMessage("clicked on appliedUnitButton%s"%(self.unitChars[inputClass.getData1()]), "")
                self.resetPromotions()
                iUnit = inputClass.getData1() -65
                iCounter = 0
                for i in self.spaceMod.getUnitAt(iUnit).getPromos():
                    self.unitPromotions[iCounter] = i
                    iCounter = iCounter + 1
                self.updateUnitPromotionList()
                self.showUnit(self.spaceMod.getUnitAt(iUnit))
                return 1
            
            # if a promotion button was pressed
            
            elif inputClass.getFunctionName() == "UnitPromotionButton%s"%(self.unitChars[inputClass.getData1()]):
                #CyInterface().addImmediateMessage("clicked on UnitPromotionButton %d"%(inputClass.getData1() - 65), "")
                self.screenValues["promotionSelected"] = inputClass.getData1() - 65
                #self.__eventSetPromotion()
                return 1
            
            return 0
        
        return 0

    def update(self, fDelta):
        "this function is continuously called every few hundred milliseconds, dont need to do anything"
        return

    def getScreen(self):
        "gets the screen instance"

        return CyGInterfaceScreen(self.SCREEN_NAME, self.screenId)
                                        
    def getNextWidgetName(self):
        "returns a unique ID for a widget in this screen"

        szName = self.WIDGET_ID + str(self.nWidgetCount)
        self.nWidgetCount += 1
        return szName
                
    def deleteAllWidgets(self):
        "deletes all widgets in the screen"      
        i = self.nWidgetCount - 1
        while (i >= 0):
            self.nWidgetCount = i
            self.screen.deleteWidget(self.getNextWidgetName())
            i -= 1

        self.nWidgetCount = 0   
                                        
    def pushScreen(self, screenQueueNum):
        "pushes a screen onto the queue so it will be showed next time showScreen is called"

        self.screenQueue.append(screenQueueNum)

    def popScreen(self):
        "pops a screen off the start of the queue"

        id = self.screenQueue.pop(0)
        return id

    def showNextScreen(self):
        "shows the next screen, returns true for a screen was in the queue to show, false if queue was empty (no more screens left)"

        if (len(self.screenQueue) != 0):
            id = self.popScreen()
            self.updateScreen(id)
            return true
        else:
            return false


    def updateScreen(self, id):
        "updates the screen by drawing all the widgets onto the screen"
        self.deleteAllWidgets()
#        self.fillUnitClasses()
#        self.fillWeaponList()
        self.showUnitBoxScreen()
            
    def showUnitBoxScreen(self):
        
        titletext = "<font=4>Unit Sandbox</font>"

        self.screen.setLabel(self.getNextWidgetName(), "", titletext, CvUtil.FONT_CENTER_JUSTIFY, self.xcentre, self.ycentre - (self.HEIGHT/2) + 10, -2.0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )

        #create body text that says where olympic games were held
        lbl_class = "<font=2>" + "UnitClass" + "</font>"
        lbl_name = "<font=2>" + "UnitName" + "</font>"
        lbl_weapon = "<font=2>" + "WeaponName" + "</font>"
        lbl_promos = "<font=2>" + "Available Promotions" + "</font>"
        
        self.screen.setLabel(self.getNextWidgetName(), "", lbl_class, CvUtil.FONT_LEFT_JUSTIFY, self.xcentre - self.WIDTH/2 + 20, self.ycentre - 240, -2.0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
        self.screen.setLabel(self.getNextWidgetName(), "", lbl_name, CvUtil.FONT_LEFT_JUSTIFY, self.xcentre - self.WIDTH/2 + 20, self.ycentre - 285, -2.0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
        self.screen.setLabel(self.getNextWidgetName(), "", lbl_weapon, CvUtil.FONT_LEFT_JUSTIFY, self.xcentre - self.WIDTH/2 + 20, self.ycentre - 190, -2.0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
        self.screen.setLabel(self.getNextWidgetName(), "", lbl_promos, CvUtil.FONT_LEFT_JUSTIFY, self.xcentre + self.WIDTH/4 + 20, self.ycentre - 265, -2.0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
        
        #create DropDownBox for Classes in the upper left corner
        if (self.screenValues["screenOpen"] == False):
            #CyInterface().addImmediateMessage("screenValues[screenOpen] is False", "")
            szDropdown01Name = str("UnitClassChoice")
            self.screen.addDropDownBoxGFC(szDropdown01Name, self.xcentre - self.WIDTH/2 + 20, self.ycentre - 225, 160, WidgetTypes.WIDGET_GENERAL, -1, -1, FontTypes.GAME_FONT)
            if (self.unitClasses == None):
                self.fillUnitClasses()
            #CyInterface().addImmediateMessage("length of unitClasses = %d" %(len(self.unitClasses)), "")
            for i in range(len(self.unitClasses)):
                #CyInterface().addImmediateMessage(unicode(self.weaponList[i]), "")
                self.screen.addPullDownString(szDropdown01Name, gc.getUnitClassInfo(self.unitClasses[i]).getDescription(), i, i, True)
        
        if (self.screenValues["weaponChanged"] == False):        
            szDropdown02Name = str("UnitWeaponChoice")
            self.screen.addDropDownBoxGFC(szDropdown02Name, self.xcentre - self.WIDTH/2 + 20, self.ycentre - 175, 160, WidgetTypes.WIDGET_GENERAL, -1, -1, FontTypes.GAME_FONT)
            if (self.weaponList == None):
                self.fillWeaponList()
            #CyInterface().addImmediateMessage("length of weaponList = %d" %(len(self.weaponList)), "")
            for i in range(len(self.weaponList)):
                #CyInterface().addImmediateMessage(unicode(self.weaponList[i]), "")
                self.screen.addPullDownString(szDropdown02Name, gc.getUnitInfo(self.weaponList[i]).getDescription(), i, i, True)
            self.screenValues["weaponChanged"] = True
        
#        unitName = gc.getUnitInfo(self.screenValues["selectedUnit"]).getDescription()
        if (self.screenValues["currentUnitName"] == None):
            self.screenValues["currentUnitName"] = gc.getUnitInfo(self.screenValues["selectedUnit"]).getDescription()
                    
        self.screen.setText( "UnitNameBox", "Background", "<font=4>" + self.screenValues["currentUnitName"] + "</font>", CvUtil.FONT_LEFT_JUSTIFY, self.xcentre - self.WIDTH/2 + 20, self.ycentre - 270, -0.3, FontTypes.GAME_FONT, WidgetTypes.WIDGET_CITY_NAME, gc.getInfoTypeForString(gc.getUnitInfo(self.screenValues["selectedUnit"]).getType()), -1 )
        self.screen.show( "UnitNameBox" )
        
        #create promotions panels and buttons
        unitIndex = 65
        panelID = "PromotionBoxPanelA"
        for i in range(self.maxPromotions):
            if (i % 2 == 0):
                panelID = "PromotionBoxPanel%s"%(self.unitChars[i+65])
                self.screen.addPanel(panelID, "Active Promotions", "", False, False, self.xcentre - self.WIDTH/2 + 20, self.ycentre - 150 + i*50, 160, 100, PanelStyles.PANEL_STYLE_BLUE50 )
            art = None
            if (self.unitPromotions[i] != -1):
                art = gc.getPromotionInfo(self.unitPromotions[i]).getButton() 
            else:
                art = gc.getTechInfo(16).getButton()
            self.screen.attachImageButton( panelID, "UnitPromotionButton%s"%(self.unitChars[unitIndex]), art, GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_GENERAL, unitIndex, -1, False )
            unitIndex = unitIndex + 1
           
      
        # create unitPanel with the graphic of the selected unit
        self.screen.addPanel( "UnitViewPanel", u"", u"", True, False, self.xcentre - self.WIDTH/4, self.ycentre - 250, self.WIDTH/2, 400, PanelStyles.PANEL_STYLE_BLUE50 )

        #CyInterface().addImmediateMessage("displaying %d - %s" %(gc.getInfoTypeForString(gc.getUnitInfo(self.screenValues["selectedUnit"]).getType()), gc.getUnitInfo(self.screenValues["selectedUnit"]).getDescription()),"")
        self.screen.addUnitGraphicGFC( "InterfaceUnitModel", gc.getInfoTypeForString(gc.getUnitInfo(self.screenValues["selectedUnit"]).getType()), self.xcentre - self.WIDTH/6, self.ycentre - 250, self.WIDTH/3, 200, WidgetTypes.WIDGET_UNIT_MODEL, gc.getInfoTypeForString(gc.getUnitInfo(self.screenValues["selectedUnit"]).getType()), -1,  -20, 30, 1, False )
        
        #create promotion list
        listID = "PromotionBoxList"
        self.screen.addListBoxGFC(listID, "", self.xcentre + self.WIDTH/4 + 20, self.ycentre - 240, 160, 390, TableStyles.TABLE_STYLE_STANDARD)
        self.screen.enableSelect(listID, True)
        self.screen.setStyle(listID, "Table_StandardCiv_Style")
        
        # add promotion list entries
        self.updateUnitPromotionList()
        for k in self.unitPromotionList: 
            #CyInterface().addImmediateMessage("attach : " + "appliedUnitButton%s"%(self.unitChars[promoIndex]), "")
            if (isPromotionValid(k, self.screenValues["selectedUnit"])):
                self.screen.appendListBoxString( listID, gc.getPromotionInfo(k).getDescription(), WidgetTypes.WIDGET_GENERAL, k, -1, CvUtil.FONT_LEFT_JUSTIFY )
                
        # add promo description lines under the graphic
        iCounter = 65
        for k in self.unitPromotions:
            if (k != -1):
                #self.screen.setLabel(self.getNextWidgetName(), "", gc.getPromotionInfo(k).getDescription(), CvUtil.FONT_CENTER_JUSTIFY, self.xcentre, self.ycentre - 100 + (iCounter-65)*20, self.WIDTH/2-40, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
                szSpecialText = CyGameTextMgr().getPromotionHelp(k, False)
                #CyInterface().addImmediateMessage("len of text : %d"%(len(szSpecialText)), "")
                self.screen.addMultilineText(self.getNextWidgetName(), szSpecialText, self.xcentre - self.WIDTH/4, self.ycentre - 80 + (iCounter-65)*40, self.WIDTH/2-40, 50, WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
                iCounter = iCounter + 1

        # create applied unit panel and unit-buttons
        self.screen.addPanel( "AppliedUnitsPanel", u"", u"", False, True, self.xcentre - self.WIDTH/2 + 20, self.ycentre + 160, self.WIDTH/10*9+40, 80, PanelStyles.PANEL_STYLE_BLUE50 )
        #CyInterface().addImmediateMessage("spaceMod unitList len : %d" %(len(self.spaceMod.getUnits())), "")
        
        unitIndex = 65
        for iUnit in self.spaceMod.getUnits(): 
            #CyInterface().addImmediateMessage("attach : " + "appliedUnitButton%s"%(self.unitChars[unitIndex]), "")
            self.screen.attachImageButton( "AppliedUnitsPanel", "appliedUnitButton%s"%(self.unitChars[unitIndex]), gc.getUnitInfo(iUnit.getType()).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_GENERAL, unitIndex, -1, False )
            unitIndex = unitIndex + 1
             
        self.screenValues["screenOpen"] = True

    def fillWeaponList(self):
        self.weaponList = []
        if (self.screenValues["selectedClass"] != None):
            for unitWeapon in self.spaceMod.getUnitsOfClass(self.screenValues["selectedClass"]):
                self.weaponList.append(unitWeapon)
            if (self.screenValues["selectedUnit"] == None):
                self.screenValues["selectedUnit"] = self.weaponList[len(self.weaponList)-1]          
            
    def fillUnitClasses(self):
        self.unitClasses = []
        for unitClass in self.spaceMod.getTrainableUnitClassesFromPlayer(gc.getPlayer(0)):
            self.unitClasses.append(unitClass)
        if (self.screenValues["selectedClass"] == None):
            self.screenValues["selectedClass"] = self.unitClasses[len(self.unitClasses)-1]
                
    def clearScreenValues(self):
        for key in self.screenValues.keys():
            self.screenValues[key] = None 
            
    def setCurrentUnitName(self, sName):
        self.screenValues["currentUnitName"] = sName
        self.updateScreen(self.screenId)
        
    def showUnit(self, vUnit):
        self.screenValues["selectedUnit"] = vUnit.getType()
        self.screenValues["currentUnitName"] = vUnit.getName()
        self.updateScreen(self.screenId)
        
    def updateUnitPromotionList(self):
        self.unitPromotionList = []
        if self.screenValues["selectedUnit"] != None:
            for k in range(gc.getNumPromotionInfos()):
                if (isPromotionValid(k, self.screenValues["selectedUnit"])):
                    self.unitPromotionList.append(k)
            for promo in self.unitPromotions:
                if (promo != -1):
                    try:
                        self.unitPromotionList.remove(promo)
                    except(ValueError):
                        continue
               
    def resetPromotions(self):
        self.unitPromotions = []
        for i in range(self.maxPromotions):
            self.unitPromotions.append(-1)
        self.updateUnitPromotionList()
            
    def __eventEditUnitRename(self, iUnit):
        popup = PyPopup.PyPopup(5001, EventContextTypes.EVENTCONTEXT_ALL)
        popup.setHeaderString("Unit Name")
        popup.setBodyString("Enter a new Unit Name")
        popup.createEditBox(gc.getUnitInfo(iUnit).getDescription())
        popup.setEditBoxMaxCharCount( 15 )
        popup.launch()       
        
    def __eventSetPromotion(self):
        # This isn't working, but anyway a nice idea to pop up a table of available promotions instead of the right list
        popup = PyPopup.PyPopup(5002, EventContextTypes.EVENTCONTEXT_ALL)
        popup.setHeaderString("Choose a Promotion")
        popup.createTable(2, 2)
        cellPos = [0, 0]
        for k in range(4):
            #popup.addTableCellDDS(cellPos[0], cellPos[1], gc.getPromotionInfo(k).getButton())
            popup.addTableCellText(cellPos[0], cellPos[1], "testCell")
            if ((k+1)%2 == 0):
                cellPos[0] = cellPos[0] + 1
                cellPos[1] = 0
            else:
                cellPos[1] = cellPos[1] + 1
                    
        popup.completeTableAndAttach(0, 0)
        popup.launch()
        
    