# SpaceMod.py
# Author : Vadus
# This is the central class of the SpaceMod. All additional functionalities are located here.
# Therefore this is a Singelton Object, so that the same instance is used in every using class


from CvPythonExtensions import *
import pydoc
import PyHelpers

import VadusUnit
import UnitBoxScreen
import CvVadusScreenUtils

import CvUtil

gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer

#SpaceMod Singleton, because the same, and only one!, instance has to be used in different classes
class SpaceMod:
    
    class __impl:
        """ Implementation of the singleton interface """
        
        def __init__(self):
            self.unitList = []
            self.playerCityBuildsUnit = {}
            self.currentUnitName = ""
            self.playerCityActionButtons = {}
        
        def getId(self):
            """ Test method, return singleton id """
            return id(self)
                  
        def onUnitBuilt(self, argsList):
            vUnitBuilt = False
            toDelete = 0
            city = argsList[0]
            unit = argsList[1]
            player = PyPlayer(city.getOwner())
            if player.getID() in self.playerCityBuildsUnit.keys():
                for cityUnit in self.playerCityBuildsUnit[player.getID()]:
                    if city.getID() == cityUnit[0]:
                        if unit.getUnitType() == cityUnit[1].getType():
                            #CyInterface().addImmediateMessage("yeah, the %s was built"%(cityUnit[1].getName()), "")
                            unit.setName(cityUnit[1].getName())
                            for promo in cityUnit[1].getPromos():
                                unit.setHasPromotion(promo, True)
                            vUnitBuilt = True
                            break
                    toDelete = toDelete + 1
                        
                if (vUnitBuilt):
                    del self.playerCityBuildsUnit[player.getID()][toDelete]
        
        def getTrainableUnitClassesFromPlayer(self, pPlayer):
            unitClasses = []
            for i in range(gc.getNumUnitClassInfos()):
                eLoopUnit = gc.getCivilizationInfo(pPlayer.getCivilizationType()).getCivilizationUnits(i)
               #get only fighting units.. don't know, if this is better for future .. 
                if (gc.getUnitInfo(eLoopUnit).getCombat() > 0):
                    if (pPlayer.canTrain(eLoopUnit, False, True)):
                        unitClasses.append(i)
            return unitClasses
        
        def getUnitsOfClass(self, iUnitClass):
            unitTypes = []
#            CyInterface().addImmediateMessage("getUnitsOfClass : %d" %(iUnitClass), "")
            for i in range(gc.getNumUnitInfos()):
                if (gc.getUnitInfo(i).getUnitClassType() == iUnitClass):
                    unitTypes.append(i)
#            CyInterface().addImmediateMessage("UnitsOfClass %d are %s" %(iUnitClass, str(unitTypes)), "")
            return unitTypes
        
        def setCurrentUnitName(self, sName):
            CvVadusScreenUtils.unitBoxScreen.setCurrentUnitName(sName)
                
        def addUnit(self, iClass, iUnitType, sUnitName, *pPromos):
            self.unitList.append(VadusUnit.VadusUnit(iClass, iUnitType, sUnitName, *pPromos))
            
        def getUnitAt(self, iIndex):
            return self.unitList[iIndex]
        
        def getUnits(self):
            return self.unitList
        
        def showUnitBoxScreen(self):
            CvVadusScreenUtils.unitBoxScreen.pushScreen(0)
            CvVadusScreenUtils.unitBoxScreen.showScreen(self)
            
        def cityProduceUnit(self, vUnit, iCity=-1):
            pHeadSelectedCity = None
            if iCity == -1:
                pHeadSelectedCity = CyInterface().getHeadSelectedCity()
            #pHeadSelectedCity.setUnitProduction(iUnit, 1)
            if (vUnit.getType() != UnitTypes.NO_UNIT):
                pHeadSelectedCity.pushOrder(OrderTypes.ORDER_TRAIN, vUnit.getType(), -1, False, False, False, False)
                if gc.getActivePlayer().getID() in self.playerCityBuildsUnit.keys():
                    self.playerCityBuildsUnit[gc.getActivePlayer().getID()].append((pHeadSelectedCity.getID(),vUnit))
                else:
                    self.playerCityBuildsUnit[gc.getActivePlayer().getID()] = [(pHeadSelectedCity.getID(),vUnit)]
            
        def setUnitProduceButtonAction(self, iPlayer, iCity, iActionNum, vUnit):
            if iPlayer in self.playerCityActionButtons.keys():
                if iCity in self.playerCityActionButtons[iPlayer].keys():
                    self.playerCityActionButtons[iPlayer][iCity].append((iActionNum, vUnit))
                else:
                    self.playerCityActionButtons[iPlayer][iCity] = [(iActionNum, vUnit)]
            else:
                self.playerCityActionButtons[iPlayer] = {iCity : [(iActionNum, vUnit)]}
        
        def getUnitProduction(self, iCity, iPlayer=-1):
            if iPlayer == -1:
                iPlayer = gc.getActivePlayer().getID()
            if iPlayer in self.playerCityBuildsUnit.keys():
                if iCity in self.playerCityActionButtons[iPlayer].keys():
                    for cityUnit in self.playerCityBuildsUnit[iPlayer]:
                        if iCity == cityUnit[0]:
                            return cityUnit[1].getName()
            return None
        
        def getUnitProduceButtonAction(self):
            return self.playerCityActionButtons
        
        def clearUnitProduceButtonActionForCity(self, iPlayer, iCity):
            if iPlayer in self.playerCityActionButtons.keys():
                if iCity in self.playerCityActionButtons[iPlayer].keys():
                    self.playerCityActionButtons[iPlayer][iCity] = []
                    
        def getUnitFromAction(self, iActionNum):
            for actionUnit in self.playerCityActionButtons[gc.getActivePlayer().getID()][CyInterface().getHeadSelectedCity().getID()]:
                if iActionNum == actionUnit[0]:
                    return actionUnit[1]
            return 0

    # storage for the instance reference
    __instance = None

    def __init__(self):
        """ Create singleton instance """
        # Check whether we already have an instance
        if SpaceMod.__instance is None:
            # Create and remember instance
            SpaceMod.__instance = SpaceMod.__impl()

        # Store instance reference as the only member in the handle
        self.__dict__['_Singleton__instance'] = SpaceMod.__instance
             
    def __getattr__(self, attr):
        """ Delegate access to implementation """
        return getattr(self.__instance, attr)

    def __setattr__(self, attr, value):
        """ Delegate access to implementation """
        return setattr(self.__instance, attr, value)

    




